from datetime import datetime
from typing import Any
from uuid import UUID

import pytest

from camunda_client.types_ import TypedVariableValueSchema


@pytest.mark.parametrize(
    ("variable_type", "value", "expected"),
    [
        (int, "888", 888),
        (int | None, "888", 888),
        (int | None, None, None),
        (
            UUID,
            "29b34aa1-0eae-42d9-9e97-8b6c028e4841",
            UUID("29b34aa1-0eae-42d9-9e97-8b6c028e4841"),
        ),
        (datetime, datetime.min.isoformat(), datetime.min),
    ],
)
def test_typed(
    variable_type: type[Any],
    value: Any,
    expected: Any,
) -> None:
    response = {"type": "String", "value": value, "valueInfo": {}}
    schema = TypedVariableValueSchema[
        variable_type  # type:ignore[valid-type]
    ].model_validate(response)
    assert isinstance(schema.value, variable_type)
    assert schema.value == expected



from .body import (
    CompleteExternalTaskSchema,
    ExtendLockOnExternalTaskSchema,
    ExternalTaskFailureSchema,
    FetchExternalTasksSchema,
    FetchExternalTaskTopicSchema,
    TaskBpmnErrorSchema,
)

__all__ = [
    "CompleteExternalTaskSchema",
    "ExtendLockOnExternalTaskSchema",
    "ExternalTaskFailureSchema",
    "FetchExternalTasksSchema",
    "FetchExternalTaskTopicSchema",
    "TaskBpmnErrorSchema",
]



from enum import Enum


class ProcessInstanceQuerySortEnum(Enum):
    INSTANCE_ID = "instanceId"
    DEFINITION_ID = "definitionId"
    DEFINITION_KEY = "definitionKey"
    BUSINESS_KEY = "businessKey"
    TENANT_ID = "tenantId"


class State(Enum):
    ACTIVE = "ACTIVE"
    SUSPENDED = "SUSPENDED"
    COMPLETED = "COMPLETED"
    EXTERNALLY_TERMINATED = "EXTERNALLY_TERMINATED"
    INTERNALLY_TERMINATED = "INTERNALLY_TERMINATED"


class TaskQuerySortEnum(Enum):
    TASK_ID = "taskId"
    ACTIVITY_INSTANCE_ID = "activityInstanceId"
    PROCESS_DEFINITION_ID = "processDefinitionId"
    PROCESS_INSTANCE_ID = "processInstanceId"
    EXECUTION_ID = "executionId"
    DURATION = "duration"
    END_TIME = "endTime"
    START_TIME = "startTime"
    TASK_NAME = "taskName"
    TASK_DESCRIPTION = "taskDescription"
    ASSIGNEE = "assignee"
    OWNER = "owner"
    DUE_DATE = "dueDate"
    FOLLOW_UP_DATE = "followUpDate"
    DELETE_REASON = "deleteReason"
    TASK_DEFINITION_KEY = "taskDefinitionKey"
    PRIORITY = "priority"
    CASE_DEFINITION_ID = "caseDefinitionId"
    CASE_INSTANCE_ID = "caseInstanceId"
    CASE_EXECUTION_ID = "caseExecutionId"
    TENANT_ID = "tenantId"


class SortOrder(Enum):
    ASC = "asc"
    DESC = "desc"


class Operator(Enum):
    EQ = "eq"
    NEQ = "neq"
    GT = "gt"
    GTEQ = "gteq"
    LT = "lt"
    LTEQ = "lteq"
    LIKE = "like"
    NOT_LIKE = "notLike"


class DelegationState(Enum):
    PENDING = "PENDING"
    RESOLVED = "RESOLVED"


class IncidentStatus(Enum):
    OPEN = "open"
    RESOLVED = "resolved"


class SortByHistoricProcessInstance(Enum):
    INSTANCE_ID = "instanceId"
    DEFINITION_ID = "definitionId"
    DEFINITION_KEY = "definitionKey"
    DEFINITION_NAME = "definitionName"
    DEFINITION_VERSION = "definitionVersion"
    BUSINESS_KEY = "businessKey"
    START_TIME = "startTime"
    END_TIME = "endTime"
    DURATION = "duration"
    TENANT_ID = "tenantId"



from typing import Any
from uuid import UUID

from pydantic import ConfigDict

from camunda_client.clients.types_ import CommaSeparatedStrList
from camunda_client.types_ import BaseSchema

from .enums import (
    Operator,
    ProcessInstanceQuerySortEnum,
    SortByHistoricProcessInstance,
    SortOrder,
)


class ConditionQueryParameterSchema(BaseSchema):
    model_config = ConfigDict(use_enum_values=True)

    operator: Operator
    value: Any


class VariableParameterSchema(ConditionQueryParameterSchema):
    name: str | None = None


class ProcessInstanceQuerySortItemSchema(BaseSchema):
    model_config = ConfigDict(use_enum_values=True)

    sort_by: ProcessInstanceQuerySortEnum | None = None
    sort_order: SortOrder | None = None


class ProcessInstanceQuerySchema(BaseSchema):
    deployment_id: str | None = None
    process_definition_id: str | None = None
    process_definition_key: str | None = None
    process_definition_key_in: list[str] | None = None
    process_definition_key_not_in: list[str] | None = None
    business_key: str | None = None
    business_key_like: str | None = None
    case_instance_id: str | None = None
    super_process_instance: str | None = None
    sub_process_instance: str | None = None
    super_case_instance: str | None = None
    sub_case_instance: str | None = None
    active: bool | None = None
    suspended: bool | None = None
    process_instance_ids: list[str] | None = None
    with_incident: bool | None = None
    incident_id: str | None = None
    incident_type: str | None = None
    incident_message: str | None = None
    incident_message_like: str | None = None
    tenant_id_in: list[str] | None = None
    without_tenant_id: bool | None = None
    process_definition_without_tenant_id: bool | None = None
    activity_id_in: list[str] | None = None
    root_process_instances: bool | None = None
    leaf_process_instances: bool | None = None
    variables: list[VariableParameterSchema] | None = None
    variable_names_ignore_case: bool | None = None
    variable_values_ignore_case: bool | None = None
    or_queries: list["ProcessInstanceQuerySchema"] | None = None
    sorting: list[ProcessInstanceQuerySortItemSchema] | None = None


class SortingHistoricProcessInstanceSchema(BaseSchema):
    sortBy: SortByHistoricProcessInstance | None = None
    sortOrder: SortOrder | None = None


class HistoryVariableInstanceFilterParamsSchema(BaseSchema):
    variable_name: str | None = None
    variable_name_like: str | None = None
    variable_name_in: CommaSeparatedStrList[str] | None = None
    variable_value: str | None = None
    process_instance_id: UUID | None = None
    process_instance_id_in: CommaSeparatedStrList[UUID] | None = None
    task_id_in: CommaSeparatedStrList[UUID] | None = None
    deserialize_values: bool | None = None
    include_deleted: bool | None = None
    tenant_id_in: CommaSeparatedStrList[str] | None = None
